#include <Adafruit_NeoPixel.h>

#define PIN 3
#define TOTAL_LEDS 8

Adafruit_NeoPixel strip = Adafruit_NeoPixel(TOTAL_LEDS, PIN, NEO_GRB + NEO_KHZ800);

const int SEG = 1000, TOTAL_MODE = 8, capThreshold = 150; 
int sel = -1;
long time_init = 0, time_last = 0, time_now = 0, duration = 0;
int r_seg = 2; // lights led per 5 second
boolean last_state = true;
boolean init_state = false;
boolean selected = false;
uint8_t rainbow7[7][3] = {
                            {255,0,0},
                            {255,60,0},
                            {200,200,0},
                            {0,255,0},
                            {0,180,180},
                            {0,0,255},
                            {120,0,120},
                          };

void setup() {
  pinMode(2, INPUT_PULLUP);
  pinMode(1, OUTPUT);
  strip.begin();
  strip.show(); // Initialize all pixels to 'off'
  
  strip.setPixelColor(0, strip.Color(255, 0, 0));
  strip.show();
  init_state = digitalRead(2);  // if change then starting counting
  time_init = millis();
  delay(10);
}

void loop() {
  boolean reading = digitalRead(2);

  if (!selected){
    if (reading == init_state){ 
      long passed = millis() - time_init;
      sel = (passed/1000) % TOTAL_MODE;  
      colorWipe(strip.Color(0, 0, 0), 0); 
      strip.setPixelColor(sel, strip.Color(255, 0, 0));
      strip.show();
    } else if (reading != init_state){
      selected = true;
    }
  } else {
    switch (sel) {
      case 0 :
        strip.setPixelColor(0, strip.Color(255, 0, 0));// charge mode
        while(1);
        break;
      case 1 :
        colorWipe(strip.Color(255, 0, 0), 10); // Green
        while(1);
        break;
      case 2 :
        colorWipe(strip.Color(0, 0, 255), 10); // Blue
        while(1);
        break;   
      case 3 :
        while(1) rgb(); // r,g,b each for 3 seconds 
        break;    
      case 4 :
        while(1) thunderCar(20);
        break;
      case 5 :
        while(1) rainbow(20);
        break;
      case 6 :
        while(1) theaterChaseRainbow(50);
        break;    
      case 7 :
        while(1) rainbowCycle(20);
        break; 
      default :
        while(1) rainbow(20);
        break;                  
    }
  }
}

void fadeOff(){
  colorWipe(strip.Color(255, 0, 0), 10); // Red

  delay(1000);
  for (int e = 0; e < 127 ; e++){
    for(uint16_t i=0; i<strip.numPixels(); i++) {
        strip.setPixelColor(i, strip.Color(255-e*2, 0, 0));       
    }
    strip.show();
    delay(5);
  }
  colorWipe(strip.Color(0, 0, 0), 0); 
}

void change(){
  colorWipe(strip.Color(255, 0, 0), 50); //Red   
   
  delay(2000);
}

// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i, c);
      strip.show();
      delay(wait);
  }
}

void rainbow(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256; j++) {
    for(i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel((i+j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256*5; j++) { // 5 cycles of all colors on wheel
    for(i=0; i< strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

//Theatre-style crawling lights.
void theaterChase(uint32_t c, uint8_t wait) {
  for (int j=0; j<10; j++) {  //do 10 cycles of chasing
    for (int q=0; q < 3; q++) {
      for (int i=0; i < strip.numPixels(); i=i+3) {
        strip.setPixelColor(i+q, c);    //turn every third pixel on
      }
      strip.show();
     
      delay(wait);
     
      for (int i=0; i < strip.numPixels(); i=i+3) {
        strip.setPixelColor(i+q, 0);        //turn every third pixel off
      }
    }
  }
}

//Theatre-style crawling lights with rainbow effect
void theaterChaseRainbow(uint8_t wait) {
  for (int j=0; j < 256; j++) {     // cycle all 256 colors in the wheel
    for (int q=0; q < 3; q++) {
        for (int i=0; i < strip.numPixels(); i=i+3) {
          strip.setPixelColor(i+q, Wheel( (i+j) % 255));    //turn every third pixel on
        }
        strip.show();
       
        delay(wait);
       
        for (int i=0; i < strip.numPixels(); i=i+3) {
          strip.setPixelColor(i+q, 0);        //turn every third pixel off
        }
    }
  }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  if(WheelPos < 85) {
   return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
  } else if(WheelPos < 170) {
   WheelPos -= 85;
   return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  } else {
   WheelPos -= 170;
   return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
}

void racing(){
  long start_time = 0;
  flash(50);  
  while(1){
    long elapsed = (millis()-start_time)/r_seg/1000;
    int tmp = 0;

    if (elapsed >= strip.numPixels()){
      theaterChase(strip.Color(127, 127, 127), 50); // White
    } else {   
      for (int i = 0; i < elapsed; i++){
        tmp = i%7;
        strip.setPixelColor(i, strip.Color(rainbow7[tmp][0], rainbow7[tmp][1], rainbow7[tmp][2]));
      }
      for (int i = elapsed; i < strip.numPixels(); i++){
        strip.setPixelColor(i, strip.Color(0, 0, 0));
      }
      strip.setBrightness(100);
      strip.show();
      delay(10);
    }
  }
}
void flash(int wait){
  colorWipe(strip.Color(0, 0, 0), 5); // Black
  delay(1000);
  
  for(int i = 0; i <= 25; i++){
      strip.setBrightness(i*10);
      strip.setPixelColor(0, strip.Color(0, 0, 255));
      strip.show();
      delay(wait); 
  }
  for(int i = 25; i >= 0; i--){
      strip.setBrightness(i*10);
      strip.setPixelColor(0, strip.Color(0, 0, 255));  
      strip.show();
      delay(wait); 
  }
}

void rgb(){
  colorWipe(strip.Color(255, 0, 0), 50); // Red
  strip.show();
  delay(3000);
  colorWipe(strip.Color(0, 255, 0), 50); // Green
  strip.show();
  delay(3000);
  colorWipe(strip.Color(0, 0, 255), 50); // Blue
  strip.show();
  delay(3000);
}

void thunderCar(int t){
  int delayTime = 50;
  int color = 0;
  byte c = 0;
  for (int k = 0; k < 3; k++){
    for(int i = 0; i < strip.numPixels()+4; i++){
     //light up 5 leds at a time with dim color
      for (int j = 0; j < 5; j++) {
        if (i-j >= 0 && i-j < strip.numPixels()) {
          c = 255-80*j >= 0 ? 255-80*j : 0;
          switch (color) {
            case 0 :
              strip.setPixelColor(i-j, strip.Color(c, 0, 0));
              break;
            case 2 :
              strip.setPixelColor(i-j, strip.Color(0, 0, c));
              break;
            case 4 :
              strip.setPixelColor(i-j, strip.Color(0, c, 0));
              break;  
          }
        }
      }
      strip.show();
      delay(delayTime);  
    }
    color++;
    color = color > 5 ? 0 : color; 
    for(int i = strip.numPixels() - 1; i >= -4 ; i--){
      //light up 5 leds at a time with dim color
      for (int j = 0; j < 5; j++) {
        if (i+j >= 0 && i+j < strip.numPixels()) {
          c = 255-80*j >= 0 ? 255-80*j : 0;
          switch (color) {
            case 1 :
              strip.setPixelColor(i+j, strip.Color(0, c, 0));
              break;
            case 3 :
              strip.setPixelColor(i+j, strip.Color(c, 0, 0));
              break;
            case 5 :
              strip.setPixelColor(i+j, strip.Color(0, 0, c));
              break;  
          }  
        }  
      }
      strip.show();
      delay(delayTime);   
    }
    color++;
    color = color > 5 ? 0 : color; 
  }
}
